#!/usr/bin/env python
#-*- coding:utf-8 -*-

from PIL import Image
from errors import InvalidSizeError, SizeTypeError


def rmnoise(im, size=5, min_nbs=2):
    """
    Remove noise pixels
    """
    data = im.load()
    width, height = im.size
    for x in range(width):
        for y in range(height):
            if data[x, y] == 255:
                continue
            else:
                nbs = count_neighbors(data, width, height, x, y, size)
                if nbs < min_nbs:
                    data[x, y] = 255
    return im


def count_neighbors(data, w, h, x, y, size=3):
    """
    Count number of black pixel in given neighbors.
    """
    nb = 0
    if not isinstance(size, int):
        raise SizeTypeError(size)
    if size < 0 or size == size / 2 * 2:
        raise InvalidSizeError(size)

    for i in range(x - ((size - 1) / 2), x + ((size - 1)) / 2):
        for j in range(y - ((size-1) / 2), y + ((size - 1)) / 2):
            if i < 0 or j < 0 or (x == i and y == j) or i >= w or j >= h:
                continue
            else:
                if data[i, j] == 1:
                    nb += 1
    return nb



#!/usr/bin/env python
#-*- coding:utf-8 -*-

import os
from PIL import Image, ImageOps

_DIR = os.path.dirname(os.path.abspath(__file__))
_MODELS_SUFFIX = "images/models/"
_MODELS = os.path.join(_DIR, _MODELS_SUFFIX)


def match(im):
    """
    Match single char image with all models, return matched value of the image.
    """
    diffs = []
    for model in os.listdir(_MODELS):
        diffs.append(match_file(im, model))
    diffs.sort()
    return diffs[0][1][0]


def match_file(im, model_name):
    """
    Match two captchas, calculate their differences, return diff and modelname.
    """
    diff = 0
    im = Image.open(im)
    model = Image.open(_MODELS + model_name)
    img = im.resize((15, 20))
    model = model.resize((15, 20))
    width, height = model.size
    imgp = img.load()
    modelp = model.load()
    for x in range(width):
        for y in range(height):
            if imgp[x, y] != modelp[x, y]:
                diff += 1
    return diff, model_name



#!/usr/bin/env python
#-*- coding:utf-8 -*-


class InvalidSizeError(ValueError):
    """Invalid Size Error.

    The exception will be raised while filter size is not applicable.
    """
    def __init__(self, size, message=" is not an valid filter size"):
        message = str(size) + message
        super(InvalidSizeError, self).__init__(message)


class SizeTypeError(TypeError):
    """Size Type Error Error.

    The exception will be raised wihle filter size is not integer.
    """
    def __init__(self, size, message="integer argument expected, got "):
        message += str(type(size))[7:-2]
        super(SizeTypeError, self).__init__(message)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#captcha url: https://auth.szu.edu.cn/cas.aspx/

from PIL import Image, ImageOps
from rmnoise import rmnoise
from edge_detection import find_edges
from match import match
import os

_DIR = os.path.dirname(os.path.abspath(__file__))
_CROPS_SUFFIX = "images/crops/"
_CROPS = os.path.join(_DIR, _CROPS_SUFFIX)


def cleanup():
    """
    clean up crops.
    """
    for img_file in os.listdir(_CROPS):
        os.remove(_CROPS + img_file)


def preprocess(im):
    """
    do preprocessing.
    """
    im = Image.open(im)
    gray = ImageOps.grayscale(im)
    biim = gray.point(lambda i: i < 230 and 1 or 255)
    return biim


def cutcaptcha(im, edges):
    """
    cut images using edges.
    """
    for e in edges:
        xim = im.crop(e)
        of = _CROPS + "char" + str(edges.index(e)) + ".png"
        xim.save(of)


def matchcapthca():
    """
    generate captcha.
    """
    captcha = ""
    for n in range(len(os.listdir(_CROPS))):
        char = match(_CROPS + "char" + str(n) + ".png")
        captcha += str(char)
    return captcha


def captcha_regonize(im):
    """
    Run captcha regonization.
    """
    cleanup()
    img = rmnoise(preprocess(im))
    edges = find_edges(img)
    cutcaptcha(img, edges)
    captcha = matchcapthca()
    return captcha

if __name__ == "__main__":
    image = "crszu/images/captcha/gencheckcode.png"
    print captcha_regonize(image)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

from PIL import Image


def find_edges(im):
    """
    Perform edge detection then find the edges.
    """
    result = []
    data = im.load()
    histx = [0 for x in range(im.size[0])]
    for x in range(im.size[0]):
        for y in range(im.size[1]):
            if data[x, y] == 1:
                histx[x] += 1

    for i in range(im.size[0]):
        if histx[i] > 0 and histx[i-1] <= 0:
            tmpx = i
        if histx[i] > 0 and histx[i+1] <= 0:
            histy = [0 for y in range(im.size[1])]
            for y in range(im.size[1]):
                for x in range(i+1)[tmpx:]:
                    if data[x, y] == 1:
                        histy[y] += 1
            for j in range(im.size[1]):
                if histy[j] > 0 and histy[j-1] <= 0:
                    tmpy1 = j
                    break
            for j in range(im.size[1])[::-1]:
                if histy[j] > 0 and histy[j+1] <= 0:
                    tmpy2 = j+1
                    break
            if i+1 - tmpx > 2 and tmpy2 - tmpy1 > 2:
                result.append((tmpx, tmpy1, i+1, tmpy2))
    return result



from setuptools import setup

setup(
    name="crszu",
    packages=["crszu"],
    version="0.0.1",
    license="MIT License",
    install_requires=["pillow"],
    description="captcha regonization for SZU authentication.",
    author="MarkNV",
    author_email="marknv1991@gmail.com",
    url="https://github.com/marknv/crszu",
    test_suite="nose.collector",
    tests_require=["nose"],
    include_package_data=True
)
